; Timex TC2048 Hi-Colour Demo
; 
; Written to demonstrate the TC2048 Hi-Colour mode in vbSpec 1.30
;
; I've referred to the colour bars as "Copper Lines" in this source
; This is completely bogus, but I'm too lazy to bother to change the
; labels and comments! Suffice to say it's a hangover from my Amiga
; using/programming days...
;
; Author: Chris Cowley <ccowley@grok.co.uk>
;
;   Date: 30/January/2002

#DEFINE DEFB    .BYTE
#DEFINE DEFW    .WORD
#DEFINE DEFS    .TEXT


                .ORG     32768
                
                LD      A,2             ; Hi-Colour Mode
                OUT     (255),A         ; Set it
                XOR     A
                IN      A,(255)         ; Check we're running on a machine that supports hi-colour
                CP      2
                JP      Z,GO_DEMO       ; Jump to the start of the demo if we are
                
                LD      HL,HICOL_MSG
PRINT_CHAR:     LD      A,(HL)
                CP      0
                RET     Z
                RST     10H
                INC     HL
                JR      PRINT_CHAR
                
HICOL_MSG:      DEFS    "This demo requires a TC2048 or  TC2068 computer."
                DEFB    0                
;                
; We're running on a machine that supports hi-colour... so begin the demo!
;                
GO_DEMO:        XOR     A               ; Attr code 0 = black on black
                OUT     (254),A         ; Black border
                CALL    FILL_ATTRS      ; Fill the entire attribute area
                

                LD      HL,LOGO_DATA    ; Start of our 'vbSpec' logo screen
                LD      DE,16384        ; Destination (display mem)
                LD      BC,6144         ; Number of bytes = 32 * 192 = 6144
                LDIR                    ; Copy it in
                
                LD      B,25            ; 25/50ths of a second (or half a second to you)
                CALL    PAUSE
                
                LD      HL,COPPER_LIST  ; Start of the copper bar list
NEXT_COPPER:    LD      B,0             ; Zero the B register
                LD      C,(HL)          ; BC = Y posn of colour bar
                LD      A,C

                CP      201
                JR      NC,COPPER_PAUSE ; Y pos >= 201 is a pause
                
                CP      192             ; Is the Y posn = 192
                JR      Z,THE_END       ; If yes, end of COPPER_TABLE reached
                
                CP      193             ; Is the Y posn = 193
                JR      Z,SET_COUNTER   ; If yes, jump to set counter
                
                CP      194             ; Is the Y posn = 194
                JR      Z,JUMP_COUNTER  ; If yes, handle the jump
                
                INC     HL
                LD      A,(HL)
                INC     HL
                CALL    DRAW_LINE
                JR      NEXT_COPPER     ; Fetch next copper line
                              
                
THE_END:        LD      B,150           ; Wait three seconds before returning
                CALL    PAUSE           ;
                
                XOR     A               ; Zero
                LD      HL,16384
                LD      DE,16385
                LD      (HL),A
                LD      BC,6143
                LDIR                    ; Clear the screen
                OUT     (255),A         ; Switch back to the standard Speccy display
                RET
                
COPPER_PAUSE:   SUB     200
                LD      B,A
                CALL    PAUSE
                INC     HL
                JR      NEXT_COPPER
                
SET_COUNTER:    INC     HL
                LD      DE,COPPER_COUNT
                LD      A,(HL)
                LD      (DE),A
                INC     HL
                JR      NEXT_COPPER
                
JUMP_COUNTER:   INC     HL
                LD      DE,COPPER_COUNT
                LD      A,(DE)
                DEC     A               ; Decrease it
                LD      (DE),A          ; Store it back in COPPER_COUNT
                CP      0
                JR      Z,END_JUMP_CTR  ; Counter is 0, don't jump back in COPPER_LIST
                LD      B,0             ;                
                LD      C,(HL)          ; BC = Jump back bytes
                AND     A               ; Clear the carry flag
                SBC     HL,BC           ; Decrease HL by BC!
END_JUMP_CTR:   INC     HL
                JR      NEXT_COPPER

; -------------------------------------------------------------------------------------------
;  FILL_ATTRS:  Fill the entire hi-colour attribute area with the attribute specified in A  
;
;   Inputs: A  = Attribute to fill
;   Output: None
; Destroys: HL,DE,BC
; -------------------------------------------------------------------------------------------
FILL_ATTRS:     LD      HL,24576        ; Extended Attribute area
                LD      DE,24577        ; Start of Area + 1
                LD      BC,6144         ; Length of area
                LD      (HL),A          ; Make the first attr 0 (black on black)
                LDIR                    ; Copy it through to the entire attr area
                RET
                
; -------------------------------------------------------------------------------------------
;  PAUSE: Wait for a while
;
;   Inputs: B  = Number of 50ths of a seconds to wait (i.e. 25 = 1/2 second, 50 = 1 second)
;   Output: None
; Destroys: B
; -------------------------------------------------------------------------------------------
PAUSE:          HALT
                DJNZ    PAUSE
                RET
                
; -------------------------------------------------------------------------------------------
;  DRAW_LINE:
;
;   Inputs: BC = Line number (0 = top line, 191 = bottom)
;           A  = Attribute value
;   Output: None
; Destroys: BC,DE
; -------------------------------------------------------------------------------------------                
DRAW_LINE:      PUSH    HL
                SLA     C               ; Multiply BC * 2
                RL      B               ; to get offset into DRAW_TABLE
DRAW_CONT:      LD      HL,DRAW_TABLE
                ADD     HL,BC           ; HL now contains address of line
                LD      B,32
                LD      E,(HL)
                INC     HL
                LD      D,(HL)
DRAW_NXT_CLMN:  LD      (DE),A          ; Set attribute value
                INC     DE
                DJNZ    DRAW_NXT_CLMN   ; Set entire row (32 bytes)
                POP     HL
                RET                
                
DRAW_TABLE:     DEFW    $6000,$6100,$6200,$6300,$6400,$6500,$6600,$6700
                DEFW    $6020,$6120,$6220,$6320,$6420,$6520,$6620,$6720
                DEFW    $6040,$6140,$6240,$6340,$6440,$6540,$6640,$6740
                DEFW    $6060,$6160,$6260,$6360,$6460,$6560,$6660,$6760
                DEFW    $6080,$6180,$6280,$6380,$6480,$6580,$6680,$6780
                DEFW    $60A0,$61A0,$62A0,$63A0,$64A0,$65A0,$66A0,$67A0
                DEFW    $60C0,$61C0,$62C0,$63C0,$64C0,$65C0,$66C0,$67C0
                DEFW    $60E0,$61E0,$62E0,$63E0,$64E0,$65E0,$66E0,$67E0
                
                DEFW    $6800,$6900,$6A00,$6B00,$6C00,$6D00,$6E00,$6F00
                DEFW    $6820,$6920,$6A20,$6B20,$6C20,$6D20,$6E20,$6F20
                DEFW    $6840,$6940,$6A40,$6B40,$6C40,$6D40,$6E40,$6F40
                DEFW    $6860,$6960,$6A60,$6B60,$6C60,$6D60,$6E60,$6F60
                DEFW    $6880,$6980,$6A80,$6B80,$6C80,$6D80,$6E80,$6F80
                DEFW    $68A0,$69A0,$6AA0,$6BA0,$6CA0,$6DA0,$6EA0,$6FA0
                DEFW    $68C0,$69C0,$6AC0,$6BC0,$6CC0,$6DC0,$6EC0,$6FC0
                DEFW    $68E0,$69E0,$6AE0,$6BE0,$6CE0,$6DE0,$6EE0,$6FE0
                
                DEFW    $7000,$7100,$7200,$7300,$7400,$7500,$7600,$7700
                DEFW    $7020,$7120,$7220,$7320,$7420,$7520,$7620,$7720
                DEFW    $7040,$7140,$7240,$7340,$7440,$7540,$7640,$7740
                DEFW    $7060,$7160,$7260,$7360,$7460,$7560,$7660,$7760
                DEFW    $7080,$7180,$7280,$7380,$7480,$7580,$7680,$7780
                DEFW    $70A0,$71A0,$72A0,$73A0,$74A0,$75A0,$76A0,$77A0
                DEFW    $70C0,$71C0,$72C0,$73C0,$74C0,$75C0,$76C0,$77C0
                DEFW    $70E0,$71E0,$72E0,$73E0,$74E0,$75E0,$76E0,$77E0
                
COPPER_COUNT:   DEFB    00              ; Holds the counter for looping effects
                
; COPPER_LIST is a list of horizontal colour bar effects in the form:-
;                 Y,ATTR    (Y=0-191 row number, ATTR=attribute value)
;               254,PVAL    (PVAL=pause in 50ths/sec)
;               192         (end of list marker)
;
;               193,(n)     Load counter with 'n'
;               194,(nn)    Decr counter and jump back 'nn' bytes in table if counter>0

; 71 White on Black, 79 White on Blue, 87 White on Red, 95 White on Magenta, 103 Green, 111 Cyan, 119 Yellow
; 112 Black on Yellow; 120 Black on Bright White
COPPER_LIST:    
                ; Introduce colour bar my scrolling it down from the top of teh screen
                DEFB    0,79,201
                DEFB    1,79,0,87,201
                DEFB    2,79,1,87,0,95,201
                DEFB    3,79,2,87,1,95,0,103,201
                DEFB    4,79,3,87,2,95,1,103,0,111,201
                DEFB    5,79,4,87,3,95,2,103,1,111,0,119,201
                DEFB    6,79,5,87,4,95,3,103,2,111,1,119,0,111,201
                DEFB    7,79,6,87,5,95,4,103,3,111,2,119,1,111,0,103,201
                DEFB    8,79,7,87,6,95,5,103,4,111,3,119,2,111,1,103,0,95,201
                DEFB    9,79,8,87,7,95,6,103,5,111,4,119,3,111,2,103,1,95,0,87,201
                
                ; After a 1 second pause, scroll the colourbar down Y=7
                DEFB    0,79, 1,87, 2,95, 3,103, 4,111, 5,119, 6,111, 7,103, 8,95, 9,87, 10,79, 250
                DEFB    0,71, 1,79, 2,87, 3,95, 4,103, 5,111, 6,119, 7,111, 8,103, 9,95, 10,87, 11,79, 210
                DEFB    1,71, 2,79, 3,87, 4,95, 5,103, 6,111, 7,119, 8,111, 9,103, 10,95, 11,87, 12,79, 209
                DEFB    2,71, 3,79, 4,87, 5,95, 6,103, 7,111, 8,119, 9,111, 10,103, 11,95, 12,87, 13,79, 207
                DEFB    3,71, 4,79, 5,87, 6,95, 7,103, 8,111, 9,119, 10,111, 11,103, 12,95, 13,87, 14,79, 203
                DEFB    4,71, 5,79, 6,87, 7,95, 8,103, 9,111, 10,119, 11,111, 12,103, 13,95, 14,87, 15,79, 201
                DEFB    5,71, 6,79, 7,87, 8,95, 9,103, 10,111, 11,119, 12,111, 13,103, 14,95, 15,87, 16,79, 201
                DEFB    6,71, 7,79, 8,87, 9,95, 10,103, 11,111, 12,119, 13,111, 14,103, 15,95, 16,87, 17,79, 208
                
                ; Close the bar into its center
                DEFB    7,71, 8,79, 9,87, 10,95, 11,103, 12,111, 13,111, 14,103, 15,95, 16,87, 17,79, 206
                DEFB    8,71, 9,79, 10,87, 11,95, 12,103, 13,103, 14,95, 15,87, 16,79, 17,71, 204
                DEFB    9,71, 10,79, 11,87, 12,95, 13,95, 14,87, 15,79, 16,71, 201                
                DEFB    10,71, 11,79, 12,87, 13,87, 14,79, 15,71, 201                                
                DEFB    11,71, 12,79, 13,79, 14,71, 201                                                
                
                ; Make it red and scroll it down to Y=21, 1 pixel per interrupt
                DEFB    12,87, 13,23, 225
                DEFB    12,71, 13,87, 14,23, 201
                DEFB    13,71, 14,87, 15,23, 201
                DEFB    14,71, 15,87, 16,23, 201                
                DEFB    15,71, 16,87, 17,23, 201
                DEFB    16,71, 17,87, 18,23, 201                
                DEFB    17,71, 18,87, 19,23, 201
                DEFB    17,71, 18,87, 19,23, 201                
                DEFB    18,71, 19,87, 20,23, 201
                DEFB    19,71, 20,87, 21,23, 201                
                DEFB    20,71, 21,87, 22,23, 201                                
                
                ; Continue the scroll down to Y = 57, at 2 pixels per interrupt
                DEFB    21,71, 22,71, 23,87, 24,23, 201
                DEFB    23,71, 24,71, 25,87, 26,23, 201
                DEFB    25,71, 26,71, 27,87, 28,23, 201                                                                
                DEFB    27,71, 28,71, 29,87, 30,23, 201                
                DEFB    29,71, 30,71, 31,87, 32,23, 201                                
                DEFB    31,71, 32,71, 33,87, 34,23, 201                                
                DEFB    33,71, 34,71, 35,87, 36,23, 201
                DEFB    35,71, 36,71, 37,87, 38,23, 201
                DEFB    37,71, 38,71, 39,87, 40,23, 201                                
                DEFB    39,71, 40,71, 41,87, 42,23, 201
                DEFB    41,71, 42,71, 43,87, 44,23, 201
                DEFB    43,71, 44,71, 45,87, 46,23, 201                
                DEFB    45,71, 46,71, 47,87, 48,23, 201                
                DEFB    47,71, 48,71, 49,87, 50,23, 201
                DEFB    49,71, 50,71, 51,87, 52,23, 201
                DEFB    51,71, 52,71, 53,87, 54,23, 201
                DEFB    53,71, 54,71, 55,87, 56,23, 201                
                DEFB    55,71, 56,71, 57,87, 58,23, 201
                DEFB    57,71, 58,71, 59,87, 60,23, 201
                DEFB    59,71, 60,71, 61,87, 62,23, 201
                DEFB    61,71, 62,71, 63,87, 64,23, 201
                DEFB    63,71, 64,71, 65,87, 66,23, 201
                DEFB    65,71, 66,71, 67,87, 68,23, 201
                DEFB    67,71, 68,71, 69,87, 70,23, 201
                DEFB    69,71, 70,71, 71,87, 72,23, 201
                DEFB    71,71, 72,71, 73,87, 74,23, 201
                DEFB    73,71, 74,71, 75,87, 76,23, 201
                DEFB    75,71, 76,71, 77,87, 78,23, 201
                DEFB    77,71, 78,71, 79,87, 80,23, 206
                
                ; And back up to Y=57
                DEFB    77,87, 78,23, 79,71, 80,71, 201
                DEFB    75,87, 76,23, 77,71, 78,71, 201
                DEFB    73,87, 74,23, 75,71, 76,71, 201
                DEFB    71,87, 72,23, 73,71, 74,71, 201
                DEFB    69,87, 70,23, 71,71, 72,71, 201
                DEFB    67,87, 68,23, 69,71, 70,71, 201
                DEFB    65,87, 66,23, 67,71, 68,71, 201
                DEFB    63,87, 64,23, 65,71, 66,71, 201
                DEFB    61,87, 62,23, 63,71, 64,71, 201
                DEFB    59,87, 60,23, 61,71, 62,71, 201
                DEFB    57,87, 58,23, 59,71, 60,71, 201
                
                ; Expand it into Red/Yellow/Green/Cyan/Cyan/Green/Yellow/Red bar,
                ; with bright at the top, dark at the bottom
                DEFB    56,87, 57,119, 58,55, 59,23, 201
                DEFB    55,87, 56,119, 57,103, 58,39, 59,55, 60,23, 201
                DEFB    54,87, 55,119, 56,103, 57,111, 58,47, 59,39, 60,55, 61,23, 230
                
                ; Set the counter for rolling to 4
                DEFB    193,4
                
                ; Alter the brightness of each row sequentially to produce a 'rolling' effect
                DEFB    54,87, 55,119, 56,103, 57,47, 58,47, 59,39, 60,55, 61,87, 204
                DEFB    54,87, 55,119, 56,39, 57,47, 58,47, 59,39, 60,119, 61,87, 204
                DEFB    54,87, 55,55, 56,39, 57,47, 58,47, 59,103, 60,119, 61,87, 204
                DEFB    54,23, 55,55, 56,39, 57,47, 58,111, 59,103, 60,119, 61,87, 204
                
                DEFB    54,23, 55,55, 56,39, 57,111, 58,111, 59,103, 60,119, 61,23, 204
                DEFB    54,23, 55,55, 56,103, 57,111, 58,111, 59,103, 60,55, 61,23, 204
                DEFB    54,23, 55,119, 56,103, 57,111, 58,111, 59,39, 60,55, 61,23, 204
                DEFB    54,87, 55,119, 56,103, 57,111, 58,47, 59,39, 60,55, 61,23, 204
                
                ; Jump back 136 bytes to the beginning of the roll
                DEFB    194,136
                          
                DEFB    193,8                                      
                ; Split the bars up/down and separate
                DEFB    53,87,54,119,55,103,56,111,57,79,58,79,59,47,60,39,61,55,62,23,203
                DEFB    52,87,53,119,54,103,55,111,56,79,57,79,58,79,59,79,60,47,61,39,62,55,63,23,203
                DEFB    51,87,52,119,53,103,54,111,55,79,56,79,57,79
                DEFB      58,79,59,79,60,79,61,47,62,39,63,55,64,23,203
                DEFB    50,87,51,119,52,103,53,111,54,79,55,79,56,79,57,15,58,15
                DEFB      59,79,60,79,61,79,62,47,63,39,64,55,65,23,208
                       
                DEFB    194,108
                
                DEFB    230
                
                ; Explode the bar up/down, splitting it across the middle
                DEFB    47,87,48,119,49,103,50,111,51,79,52,79,53,79,54,15,55,71,56,71,57,71
                DEFB    58,71,59,71,60,71,61,15,62,79,63,79,64,79,65,47,66,39,67,55,68,23,201
                DEFB    44,87,45,119,46,103,47,111,48,79,49,79,50,79,51,15,52,71,53,71,54,71
                DEFB    61,71,62,71,63,71,64,15,65,79,66,79,67,79,68,47,69,39,70,55,71,23,201
                DEFB    41,87,42,119,43,103,44,111,45,79,46,79,47,79,48,15,49,71,50,71,51,71
                DEFB    64,71,65,71,66,71,67,15,68,79,69,79,70,79,71,47,72,39,73,55,74,23,201                
                DEFB    38,87,39,119,40,103,41,111,42,79,43,79,44,79,45,15,46,71,47,71,48,71
                DEFB    67,71,68,71,69,71,70,15,71,79,72,79,73,79,74,47,75,39,76,55,77,23,201
                DEFB    35,87,36,119,37,103,38,111,39,79,40,79,41,79,42,15,43,71,44,71,45,71
                DEFB    70,71,71,71,72,71,73,15,74,79,75,79,76,79,77,47,78,39,79,55,80,23,201
                DEFB    32,87,33,119,34,103,35,111,36,79,37,79,38,79,39,15,40,71,41,71,42,71
                DEFB    73,71,74,71,75,71,76,15,77,79,78,79,79,79,80,47,81,39,82,55,83,23,201
                DEFB    29,87,30,119,31,103,32,111,33,79,34,79,35,79,36,15,37,71,38,71,39,71
                DEFB    76,71,77,71,78,71,79,15,80,79,81,79,82,79,83,47,84,39,85,55,86,23,201
                DEFB    26,87,27,119,28,103,29,111,30,79,31,79,32,79,33,15,34,71,35,71,36,71
                DEFB    79,71,80,71,81,71,82,15,83,79,84,79,85,79,86,47,87,39,88,55,89,23,201
                DEFB    23,87,24,119,25,103,26,111,27,79,28,79,29,79,30,15,31,71,32,71,33,71
                DEFB    82,71,83,71,84,71,85,15,86,79,87,79,88,79,89,47,90,39,91,55,92,23,201
                DEFB    20,87,21,119,22,103,23,111,24,79,25,79,26,79,27,15,28,71,29,71,30,71
                DEFB    85,71,86,71,87,71,88,15,89,79,90,79,91,79,92,47,93,39,94,55,95,23,201
                DEFB    17,87,18,119,19,103,20,111,21,79,22,79,23,79,24,15,25,71,26,71,27,71
                DEFB    88,71,89,71,90,71,91,15,92,79,93,79,94,79,95,47,96,39,97,55,98,23,201
                DEFB    14,87,15,119,16,103,17,111,18,79,19,79,20,79,21,15,22,71,23,71,24,71
                DEFB    91,71,92,71,93,71,94,15,95,79,96,79,97,79,98,47,99,39,100,55,101,23,201
                DEFB    11,87,12,119,13,103,14,111,15,79,16,79,17,79,18,15,19,71,20,71,21,71
                DEFB    94,71,95,71,96,71,97,15,98,79,99,79,100,79,101,47,102,39,103,55,104,23,201
                DEFB    9,87,10,119,11,103,12,111,13,79,14,79,15,79,16,15,17,71,18,71
                DEFB    97,71,98,71,99,15,100,79,101,79,102,79,103,47,104,39,105,55,106,23,201
                DEFB    7,87,8,119,9,103,10,111,11,79,12,79,13,79,14,15,15,71,16,71
                DEFB    99,71,100,71,101,15,102,79,103,79,104,79,105,47,106,39,107,55,108,23,201
                DEFB    5,87,6,119,7,103,8,111,9,79,10,79,11,79,12,15,13,71,14,71
                DEFB    101,71,102,71,103,15,104,79,105,79,106,79,107,47,108,39,109,55,110,23,201
                DEFB    4,87,5,119,6,103,7,111,8,79,9,79,10,79,11,15,12,71
                DEFB    103,71,104,71,105,15,106,79,107,79,108,79,109,47,110,39,111,55,112,23,201
                DEFB    3,87,4,119,5,103,6,111,7,79,8,79,9,79,10,15,11,71
                DEFB    105,71,106,71,107,15,108,79,109,79,110,79,111,47,112,39,113,55,114,23,201
                DEFB    2,87,3,119,4,103,5,111,6,79,7,79,8,79,9,15,10,71
                DEFB    107,71,108,71,109,15,110,79,111,79,112,79,113,47,114,39,115,55,116,23,201
                DEFB    1,87,2,119,3,103,4,111,5,79,6,79,7,79,8,15,9,71
                DEFB    109,71,110,71,111,15,112,79,113,79,114,79,115,47,116,39,117,55,118,23,201
                DEFB    0,87,1,119,2,103,3,111,4,79,5,79,6,79,7,15,8,71
                DEFB    111,71,112,71,113,15,114,79,115,79,116,79,117,47,118,39,119,55,120,23,201
                DEFB    113,71,114,15,115,79,116,79,117,79,118,47,119,39,120,55,121,23,201
                DEFB    114,71,115,15,116,79,117,79,118,79,119,47,120,39,121,55,122,23,201
                DEFB    115,71,116,15,117,79,118,79,119,79,120,47,121,39,122,55,123,23,201                
                DEFB    116,71,117,15,118,79,119,79,120,79,121,47,122,39,123,55,124,23,201
                                
                DEFB    230
                
                ; Some foreground effects on the vbSpec section
                DEFB    48,86,49,86,212
                DEFB    47,86,48,70,49,6,50,86,203
                DEFB    46,86,47,1,50,65,51,86,203
                DEFB    45,86,46,1,51,65,52,86,203                
                DEFB    44,86,45,1,52,65,53,86,203
                DEFB    43,86,44,1,53,65,54,86,203
                DEFB    42,86,43,1,54,65,55,86,203
                DEFB    41,86,42,1,55,65,56,86,203
                DEFB    40,86,41,1,56,65,57,86,203
                DEFB    39,86,40,1,57,65,58,86,203                                
                DEFB    38,86,39,1,58,65,59,86,203
                DEFB    37,86,38,1,59,65,60,86,203
                DEFB    36,86,37,1,60,65,61,86,203
                DEFB    35,86,36,1,61,65,62,86,203
                DEFB    34,86,35,1,62,65,63,86,203
                DEFB    33,86,34,1,63,65,64,86,203
                DEFB    32,86,33,1,64,65,65,86,203
                DEFB    31,86,32,1,65,65,66,86,203
                DEFB    30,86,31,1,66,65,67,86,203
                DEFB    29,86,30,1,67,65,68,86,203
                DEFB    28,86,29,1,68,65,69,86,203
                DEFB    27,86,28,1,69,65,70,86,203
                DEFB    26,86,27,1,70,65,71,86,203
                DEFB    25,86,26,1,71,65,72,86,203
                DEFB    24,86,25,1,72,65,73,86,203
                DEFB    23,86,24,1,73,65,74,86,203
                DEFB    22,86,23,1,74,65,75,86,203
                DEFB    21,71,22,1,75,65,76,86,203
                DEFB    76,65,77,86,203
                DEFB    77,65,78,86,203
                DEFB    78,65,79,86,203
                DEFB    79,65,80,86,203
                DEFB    80,65,81,71,203
                
                ; Colourise the "Timex TC2048 Demo" text
                DEFB    100,6,101,66,102,6,103,66,104,6,105,66,106,6,107,66,108,6,109,66,110,6,111,66,201
                
                ; Colourise the (c)2002 Grok Developments Text
                DEFB    146,70,147,70,148,70,149,70,150,6,151,6,152,6,153,6
                
                ; Colourise the grok URL
                DEFB    181,71,182,71,183,71,184,71,185,71,186,7,187,7,188,7,189,7
                
                DEFB    193,12
                ; Colourise the "By Chris Cowley" text (mid point = 136, 132-141)
                DEFB    132,65,133,66,134,70,135,68,136,69,137,71,138,65,139,66,140,70,141,68,206
                DEFB    132,66,133,70,134,68,135,69,136,71,137,65,138,66,139,70,140,68,141,65,206
                DEFB    132,70,133,68,134,69,135,71,136,65,137,66,138,70,139,68,140,65,141,66,206
                DEFB    132,68,133,69,134,71,135,65,136,66,137,70,138,68,139,65,140,66,141,70,206
                DEFB    132,69,133,71,134,65,135,66,136,70,137,68,138,65,139,66,140,70,141,68,206
                DEFB    132,71,133,65,134,66,135,70,136,68,137,65,138,66,139,70,140,68,141,69,206
                DEFB    194,126
                
                DEFB    250
                DEFB    250
                DEFB    250
                DEFB    250
                
                DEFB    192             ; End of COPPER_LIST
                
                
; This is the raw screen data for the vbSpec logo screen
; Why the fsck doesn't TASM have an .INCBIN directive? Grrrr!
LOGO_DATA:      DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,7,254,0,0,15,255,255,255,128
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,1,255,1,255,7,255,143,254,3,255,255,248,0
                DEFB    255,241,255,192,127,225,255,128,127,241,255,192,0,0,0,0
                DEFB    0,0,0,0,63,239,240,7,254,3,255,15,252,0,255,224
                DEFB    255,192,127,225,255,255,255,225,255,128,0,0,0,0,0,0
                DEFB    0,0,0,0,3,255,128,7,255,255,252,3,255,255,255,0
                DEFB    255,255,255,128,63,255,255,0,63,255,255,128,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,7,255,128,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,7,254,0,0,15,255,7,255,192
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,255,131,254,7,255,7,254,1,255,255,252,0
                DEFB    255,224,255,192,255,192,255,192,255,192,255,192,0,0,0,0
                DEFB    0,0,0,0,31,239,240,7,254,3,255,31,252,0,255,224
                DEFB    255,192,127,225,255,255,255,225,255,128,0,0,0,0,0,0
                DEFB    0,0,0,0,3,255,128,7,253,255,248,1,255,255,254,0
                DEFB    255,255,255,0,31,255,254,0,31,255,255,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,7,254,0,0,0,63,255,240,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,7,254,63,128,15,254,3,255,192
                DEFB    0,3,240,0,0,255,192,0,0,255,192,0,0,0,0,0
                DEFB    0,0,0,0,255,131,254,7,255,7,255,0,255,255,255,0
                DEFB    255,224,255,192,255,192,255,192,255,192,127,224,0,0,0,0
                DEFB    0,0,0,0,31,239,224,7,254,3,255,31,254,0,255,224
                DEFB    255,192,127,225,255,128,0,1,255,128,0,0,0,0,0,0
                DEFB    0,0,0,0,1,255,0,7,252,255,240,0,255,255,252,0
                DEFB    255,255,254,0,15,255,252,0,15,255,254,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,7,254,0,0,0,255,255,252,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,7,254,0,255,199,254,255,224,15,254,1,255,192
                DEFB    255,143,252,0,3,255,248,0,3,255,248,0,0,0,0,0
                DEFB    0,0,0,0,255,131,252,7,254,3,255,0,63,255,255,128
                DEFB    255,192,127,225,255,128,127,193,255,192,124,0,0,0,0,0
                DEFB    0,0,0,0,15,255,224,7,254,3,255,15,254,1,255,224
                DEFB    255,192,127,225,255,128,0,1,255,128,124,0,0,0,0,0
                DEFB    0,0,0,0,1,255,0,7,252,127,224,0,63,255,240,0
                DEFB    255,223,252,0,7,255,248,0,7,255,248,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,7,254,0,0,1,255,255,254,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,3,254,0,255,135,255,255,240,15,254,0,0,0
                DEFB    255,159,254,0,15,255,252,0,15,255,254,0,0,0,0,0
                DEFB    0,0,0,0,127,199,252,7,254,3,255,0,15,255,255,192
                DEFB    255,192,127,225,255,128,127,225,255,128,0,0,0,0,0,0
                DEFB    0,0,0,0,15,255,224,7,255,7,254,15,255,131,255,224
                DEFB    255,224,255,224,255,192,0,0,255,192,127,224,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,31,128,0,7,255,128,0
                DEFB    255,195,240,0,0,255,192,0,0,255,192,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,7,254,0,0,3,255,255,255,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,3,254,0,255,135,255,255,248,15,255,128,0,0
                DEFB    255,191,255,0,31,255,254,0,31,255,255,0,0,0,0,0
                DEFB    0,0,0,0,127,199,252,7,254,3,255,0,1,255,255,192
                DEFB    255,192,127,225,255,255,255,225,255,128,0,0,0,0,0,0
                DEFB    0,0,0,0,7,255,192,7,255,7,254,15,255,255,255,192
                DEFB    255,224,255,192,255,192,248,0,255,192,255,224,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    255,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,7,254,0,0,7,255,255,255,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,3,255,1,255,135,255,255,252,7,255,252,0,0
                DEFB    255,255,255,128,63,255,255,0,63,255,255,128,0,0,0,0
                DEFB    0,0,0,0,63,199,248,7,254,3,255,0,0,15,255,224
                DEFB    255,192,127,225,255,255,255,225,255,128,0,0,0,0,0,0
                DEFB    0,0,0,0,7,255,192,7,255,143,254,7,255,255,255,192
                DEFB    255,241,255,192,127,225,255,192,255,225,255,192,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    255,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,7,254,0,0,7,255,255,255,128
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,1,255,1,255,7,255,255,254,7,255,255,192,0
                DEFB    255,255,255,128,127,255,255,128,127,255,255,128,0,0,0,0
                DEFB    0,0,0,0,63,199,248,7,254,3,255,0,0,1,255,224
                DEFB    255,192,127,225,255,255,255,225,255,128,0,0,0,0,0,0
                DEFB    0,0,0,0,7,255,128,7,255,255,252,7,255,255,255,128
                DEFB    255,255,255,192,127,255,255,128,127,255,255,192,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    255,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    255,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,120,0,31,15,60,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,140,64,0,0,0,0,8,196,192,76,102,48,113,136,6
                DEFB    12,0,24,8,0,192,0,0,0,6,6,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    255,192,0,0,0,0,0,0,0,0,120,0,126,1,224,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,120,0,7,143,60,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,12,28,238,112,113,238,0,192,192,12,102,48,177,240,6
                DEFB    12,224,24,0,240,195,199,57,220,6,6,28,119,56,120,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    255,192,0,0,0,0,0,0,0,0,120,0,255,7,248,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,120,0,7,143,60,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,12,12,115,152,136,196,0,192,192,0,198,49,48,120,7
                DEFB    252,96,24,1,152,198,99,24,236,6,6,34,57,204,204,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    255,192,0,0,0,0,0,0,0,0,248,1,255,135,248,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,120,121,231,143,60,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,12,12,99,25,140,104,0,192,192,1,134,50,48,156,6
                DEFB    12,96,24,3,12,204,51,24,192,6,6,99,49,141,134,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    255,192,0,0,0,0,0,0,0,3,248,1,231,143,60,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,120,121,255,135,248,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,12,12,99,25,252,48,0,192,192,66,38,51,249,4,6
                DEFB    12,103,216,11,12,204,51,24,192,6,6,127,49,141,134,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,7,248,0,7,143,60,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,120,120,255,7,248,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,255,192,0,0,0,0,15,252,30,67,193,192,16,120,15
                DEFB    30,0,3,200,1,192,0,0,0,31,240,0,0,0,0,0
                DEFB    0,12,12,99,25,128,88,0,192,96,196,38,48,49,4,6
                DEFB    12,96,12,27,12,204,51,24,192,6,12,96,49,141,134,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,7,120,0,31,15,60,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,120,120,126,1,224,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,204,204,0,0,0,0,12,204,49,204,227,96,49,140,6
                DEFB    12,96,6,56,0,192,0,0,0,6,24,0,0,0,0,0
                DEFB    0,12,12,99,24,204,140,0,192,33,135,227,96,49,136,6
                DEFB    12,96,4,49,152,198,99,56,192,6,24,51,49,140,204,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,4,120,0,30,15,60,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,140,76,0,0,0,0,8,196,96,200,102,48,113,4,6
                DEFB    12,96,12,24,0,192,0,0,0,6,12,0,0,0,0,0
                DEFB    0,63,30,247,188,121,222,3,240,31,15,225,192,120,240,15
                DEFB    30,240,3,224,241,227,193,221,240,31,240,30,123,222,120,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,17,40,32,69,22
                DEFB    4,8,170,175,148,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,133,230,51,192,244,100,135,135
                DEFB    16,233,156,146,116,151,15,78,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,201,2,3,32,2,4,8,48,137,18,4,8,132,8,0
                DEFB    152,194,36,128,17,34,0,137,32,132,76,144,50,68,136,64
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,17,40,32,69,17
                DEFB    132,8,170,168,20,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,0
                DEFB    0,8,0,0,0,0,0,2,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,120,0,0,0,0,0,0,0
                DEFB    0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,137,2,2,32,2,4,8,32,249,241,196,8,132,8,0
                DEFB    136,130,39,0,16,34,0,137,192,132,136,142,34,124,128,64
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,17,16,16,69,16
                DEFB    130,8,145,40,136,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,121,198,51,128,224,4,7,128
                DEFB    0,8,0,0,0,0,8,2,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,16,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,137,18,34,32,4,8,8,32,129,0,36,73,132,8,0
                DEFB    136,130,36,128,16,34,0,153,33,2,136,129,34,64,128,128
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,64,0
                DEFB    0,0,0,32,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,30,16,15,69,23
                DEFB    1,231,17,39,8,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,132,41,72,65,16,4,4,64
                DEFB    0,8,0,0,0,32,8,130,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    1,220,225,195,195,4,8,30,120,120,243,195,134,207,30,48
                DEFB    121,225,205,198,15,28,48,111,113,3,31,30,60,60,120,128
                DEFB    0,0,0,0,0,0,0,0,0,0,0,30,0,15,64,16
                DEFB    1,224,0,32,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,32,0,0,0
                DEFB    0,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,1,50,41,72,66,5,100,132,38
                DEFB    68,201,156,236,103,55,8,206,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,2,0,8,16,0,0,0,0,0,0,0,0,0
                DEFB    8,0,0,0,0,0,0,0,2,0,0,0,32,0,1,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,17,0,16,64,0
                DEFB    2,0,0,32,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,32,0,0,0
                DEFB    0,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,1,66,73,72,130,118,149,4,41
                DEFB    41,42,82,146,148,164,8,146,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    1,128,0,0,0,0,129,6,0,0,0,0,0,3,6,0
                DEFB    0,0,12,0,0,0,0,3,0,32,24,0,0,0,0,16
                DEFB    0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    112,0,0,0,0,0,0,0,0,0,0,0,112,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,17,68,32,121,83
                DEFB    132,7,36,167,34,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,1,66,137,73,2,20,150,4,47
                DEFB    41,234,82,146,244,162,8,146,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,129,2,0,0,1,2,8,0,0,0,4,0,4,8,0
                DEFB    0,0,4,0,0,0,0,1,0,64,8,0,0,0,0,32
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,30,68,32,69,148
                DEFB    4,8,164,168,162,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,1,51,9,74,1,20,149,4,72
                DEFB    41,10,82,146,132,161,8,146,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,179,231,198,195,1,2,30,108,112,225,239,153,143,30,0
                DEFB    109,177,197,192,15,28,1,153,112,78,235,15,108,56,120,32
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                DEFB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

.END